<?php

use Behat\Behat\Context\ClosuredContextInterface,
    Behat\Behat\Context\TranslatedContextInterface,
    Behat\Behat\Context\BehatContext,
    Behat\Behat\Exception\PendingException,
    Behat\Mink\Exception\ElementNotFoundException,
    Behat\Behat\Context\Step;
use Behat\Gherkin\Node\PyStringNode,
    Behat\Gherkin\Node\TableNode;

use Behat\MinkExtension\Context\MinkContext;
use Behat\Mink\Session;

//
// Require 3rd-party libraries here:
//
//   require_once 'PHPUnit/Autoload.php';
//   require_once 'PHPUnit/Framework/Assert/Functions.php';
//

/**
 * Features context.
 */
class DrupalContext extends MinkContext {

  /**
   * Initializes context.
   * Every scenario gets it's own context object.
   *
   * @param array $parameters context parameters (set them up through behat.yml)
   */
  public function __construct(array $parameters) {

    $this->parameters = $parameters;

    // Initialize your context here.
    $_SERVER['PHP_SELF'] = basename(__FILE__);
    $_SERVER['REMOTE_ADDR'] = $parameters['site'];
    $_SERVER['HTTP_HOST'] = $parameters['site'];
    $_SERVER['REQUEST_METHOD'] = 'GET';

    if (!defined('DRUPAL_ROOT')) {
      define('DRUPAL_ROOT', getcwd());
    }

    require_once DRUPAL_ROOT . '/includes/bootstrap.inc';
    drupal_bootstrap(DRUPAL_BOOTSTRAP_FULL);
  }

  /**
   * Returns Mink session.
   *
   * @param string|null $name name of the session OR active session will be used
   *
   * @return Session
   */
  public function getSession($name = NULL) {
    $session = $this->getMink()->getSession($name);

    // This forces the creation of a test version
    // of the database for use with behat.
    // Since it hooks into Simpletest it must use
    // simpletests naming convention.
    $ua = drupal_generate_test_ua('simpletest65');
    $session->setRequestHeader('User-Agent', $ua);

    return $session;
  }

  /** @BeforeSuite */
  public static function prepareForTheSuite($event) {
    behat_testing_setup();
    $parameters = $event->getContextParameters();

    // If configuration requires the test database to be rebuilt
    // do so here.
    if ($parameters['use_site_clone'] == 1) {
      // Clone the existing site.
      behat_testing_copy_database('simpletest65');
    }
    else {
      // Clean up any previous test runs.
      behat_testing_clean_database();
    }

  }

  /** @BeforeFeature */
  public static function beforeFeature($event) {
    $parameters = $event->getContextParameters();

    // Reinstall site.
    if (isset($parameters['rebuild_on_feature']) && !empty($parameters['rebuild_on_feature'])) {

      behat_testing_clean_database();
      behat_testing_reinstall($parameters['rebuild_on_feature']);

      if (isset($parameters['rebuild_module']) && !empty($parameters['rebuild_module'])) {
        module_enable(explode(',', $parameters['rebuild_module']));
      }
    }
  }

  /** @BeforeStep */
  public static function prepareForTheStep() {
  }

  /**
   * @Given /^I am using a test database$/
   */
  public function iAmUsingATestDatabase() {
  }


  /**
   * @Given /^I am logged in with the role "([^"]*)"$/
   */
  public function iAmLoggedInWithTheRole($role) {
    static $users;

    if (!isset($users[$role])) {
      if ($role == 'admin') {
        $users[$role] = 1;
      }
      else {
        $r = user_role_load_by_name($role);

        if (empty($r)) {
          throw new Exception(t('No such role @role', array('@role' => $role)));
        }

        $users[$role] = behat_testing_create_user(array($r))->uid;
        if ($users[$role] == FALSE) {
          throw new Exception(t('Could not create user with role @role', array('@role' => $role)));
        }
      }
    }

    // @TODO: This is a security flaw.
    return array(
      new Step\Given('I am on "/behat-testing/quick-switch/' . $users[$role] . '"'),
    );
  }

  /**
   * @Given /^I should see a "([^"]*)" field with id "([^"]*)"$/
   */
  public function iShouldSeeAFieldWithId($field_type, $id) {
    $element = $this->assertSession()->elementExists('css', $field_type . '#' . $id);
  }

  /**
   * @Then /^I should see nodes of type "([^"]*)" inside "([^"]*)"$/
   */
  public function iShouldSeeNodesOfTypeInside($type, $selector) {
    $element = $this->assertSession()->elementExists('css', $selector);
    $types = $element->find('css', '.node-' . $type);
    if (null === $types) {
      throw new ElementNotFoundException($this->getSession(), 'element', $type, $selector);
    }
    return $types;
  }

  /**
   * @Given /^I go to the "([^"]*)" node form$/
   */
  public function iGoToTheNodeForm($nodeform) {
    list($operation, $type) = explode(" ", $nodeform);
    if ($operation == 'add') {
      return array(
        new Step\Given('I am on "node/add/' . $type . '"')
      );
    }
  }

  /**
   * @Then /^I should see nodes of type "([^"]*)" inside "([^"]*)" with taxonomy "([^"]*)" matching "([^"]*)"$/
   */
  public function iShouldSeeNodesOfTypeInsideWithTaxonomyMatching($type, $selector, $node_property, $matching) {
    $node = $this->iShouldSeeNodesOfTypeInside($type, $selector);
    $node = explode('-', ($node->getAttribute('id')));
    $nid = $node[1];
    $n = node_load($nid);
    $terms = field_view_field('node', $n, $node_property);
    foreach ($terms AS $id => $term) {
      if (is_numeric($id)) {
        if ($term['#title'] == $matching) {
          return TRUE;
        }
      }
    }
    throw new ElementNotFoundException($this->getSession(), 'element', $type, $selector);
  }

  /**
   * @Given /^I am on the view "([^"]*)" with the arguments "([^"]*)"$/
   */
  public function iAmOnTheViewWithTheArgument($view, $args) {
    list($view_name, $display) = explode('|', $view);
    $v = views_get_view($view_name);
    $url = $v->display[$display]->display_options['path'];

    if ($url) {
      return new Step\Given('I am on "/' . $url . '/' . $args . '"');
    }
  }

  /**
   * @Given /^a node of type "([^"]*)" with the title "([^"]*)" and language "([^"]*)" exists$/
   */
  public function aNodeOfTypeWithTheTitleAndLanguageExists($content_type, $title, $language) {
    $account = user_load(1);
    $query = new EntityFieldQuery();
    $result = $query->entityCondition('entity_type', 'node')
      ->entityCondition('bundle', $content_type)
      ->propertyCondition('title', $title)
      ->propertyCondition('language', $language)
      ->addMetaData('account', $account)
      ->execute();

    $nodes = isset($result['node']) ? entity_load('node', array_keys($result['node'])) : array();

    if (empty($nodes)) {
      $node = new stdClass();
      $node->type = $content_type;
      $node->title = $title;
      $node->language = $language;
      node_save($node);
    }
  }

  /**
   * @Given /^a node of type "([^"]*)" with the title "([^"]*)" exists$/
   */
  public function aNodeOfTypeWithTheTitleExists($content_type, $title) {
    return $this->aNodeOfTypeWithTheTitleAndLanguageExists($content_type, $title, 'und');
  }

  /**
   * @Given /^cron has run$/
   */
  public function cronHasRun() {
    return new Step\Given('I am on "/cron.php?cron_key=' . variable_get('cron_key', 'drupal') . '"');
  }

  /**
   * @Given /^I am not logged in$/
   */
  public function iAmNotLoggedIn() {
    $this->getSession()->reset();
  }

    /**
     * @Given /^I should see a "([^"]*)" field with id "([^"]*)" and value "([^"]*)"$/
     */
    public function iShouldSeeAFieldWithIdAndValue($field_type, $id, $value) {
      $page = $this->getSession()->getPage();
      $field = $page->find('css', $field_type . '#' . $id);
      if ($field->getValue() != $value) {
        throw new Exception("$field_type with id $id does not equal $value. Found '" . $field->getValue() . "' instead");
      }
    }

    /**
     * @Given /^there should be a form error on the element with id "([^"]*)"$/
     */
    public function thereShouldBeAFormErrorOnTheElementWithId($id) {
      $page = $this->getSession()->getPage();
      $field = $page->find('css', '#' . $id);
      if (empty($field)) {
        throw new Exception('No element with id ' . $id);
      }

      if (!$this->elementHasClass($field, 'error')) {
        throw new Exception("The element with id $id has not been flagged as in an error state");
      }
    }

    /**
     * Helper function to determine if an element has a class.
     */
    public function elementHasClass($element, $class) {
      $classes = explode(' ', $element->getAttribute('class'));
      return in_array($class, $classes);
    }

}
